# -*- coding: UTF-8 -*-

import json
import logger
import propertyReader
import os
import status
import sys
import traceback
import ujson
from importlib import import_module
from artix.common.progress import Progress
from artix.filequeue.filequeuemanager import FileQueueManager

log = logger.getLogger()

#
# Загрузка справочника формата ArtixInterchange
#
unloadScript = propertyReader.unloadScript

curPath = sys.path[0]
sys.path[0] = '%s' % os.path.dirname(unloadScript)
try:
    unloadModule = __import__(os.path.basename(os.path.splitext(unloadScript)[0]), fromlist='q')
except Exception as e:
    log.error('Ошибка при подключении модуля выгрузки продаж. %s' % str(e))
sys.path[0] = curPath


def unload(path, mode, dates=None, shifts=None, closedShifts=0, saleStateId=None, unloadingShopId=None, unloadingCashCode=None):
    log.info('Запускаем скрипт выгрузки продаж формата ArtixInterchange')
    if not unloadModule:
        messEn = 'Error importing module to unload sales.'
        messRu = 'Ошибка при подключении модуля выгрузки продаж.'
        log.error(messRu)
        return status.errorUnload, messEn, messRu
    if not path:
        messEn = 'No path for file upload sales'
        messRu = 'Не указан путь для файла выгрузки продаж'
        log.error(messRu)
        return status.errorUnload, messEn, messRu
    elif not dates and not shifts:
        messEn = 'Not specified range shifts or time range unloading sales'
        messRu = 'Время выгрузки продаж или диапазон смен не указаны.'
        log.error(messRu)
        return status.errorUnload, messEn, messRu
    else:
        statusServer = '%s:%s' % (propertyReader.remoteIp, propertyReader.cashServerRestPort)
        logmess = u'Запуск выгрузки продаж'
        if mode == 'server':
            if unloadingShopId:
                logmess += u' магазин = %s' % unloadingShopId
            if unloadingCashCode:
                logmess += u' касса = %s' % unloadingCashCode
        if dates:
            logmess += u' с %s по %s в файл %s' % (dates.split('@')[0], dates.split('@')[1], path.encode('utf8'))
        else:
            logmess += u' для смен с %s по %s в файл %s' % (shifts.split('@')[0], shifts.split('@')[1], path.encode('utf8'))
        log.info(logmess)
        try:
            curPath = sys.path[0]
            sys.path[0] = '%s/../../../' % os.path.dirname(unloadScript)
            if mode == 'server':
                result = unloadModule.mainRun(unloadingShopId, unloadingCashCode, shifts, dates, converterName='aif', dataPath=path,
                                              cashUnloadStateId=saleStateId, statusHost=statusServer, closedShifts=closedShifts)
            else:
                result = unloadModule.mainRun(shifts, dates, converterName='aif', dataPath=path, cashUnloadStateId=saleStateId,
                                              statusHost=statusServer, closedShifts=closedShifts)

            sys.path[0] = curPath
            messEn = 'Unload result - %s' % result
            messRu = 'Результат выгрузки продаж - %s' % result
            log.info(messRu)
            return (status.OK if result else status.busy), messEn, messRu
        except Exception as e:
            messEn = 'Unable to unload sales. %s' % str(e)
            messRu = 'Не удалось выгрузить продажи. %s' % str(e)
            log.error(messRu)
            log.error(traceback.format_exc())
            return status.errorUnload, messEn, messRu


def filterKeys(d):
    lowerD = {}
    for k in d.keys():
        if d[k] is not None and d[k] != []:
            lowerD[k.lower()] = d[k]
    return lowerD


def jsonWrapper(jsonPiece):
    return json.loads(jsonPiece, object_hook=filterKeys)


def uploadWithoutQueue(fileName, dictProperties):
    """
    Напрямую вызывает конвертер aif для дальнейшей загрузки справочников
    без очереди. Указываем где лежит файл, передаем настройки
    """
    log.info(u'Начинаем загружать справочники напрямую в базу данных.')
    uploadScript = propertyReader.uploadScript
    curPath = sys.path[0]
    sys.path[0] = os.path.dirname(uploadScript)
    try:
        uploadModule = __import__(os.path.basename(os.path.splitext(uploadScript)[0]), fromlist='q')
        result, errorList = uploadModule.insertInDB(propertyReader, fileName, dictProperties)
        if not result:
            raise Exception
    except Exception as e:
        messRu = 'Не удалось загрузить справочники. %s' % str(e)
        log.error(messRu)
        return status.errorLoadDict, messRu
    sys.path[0] = curPath

    messRu = 'Справочники загружены'
    log.info(messRu)
    return status.OK, messRu


def getAifChunk(version, fileName, chunkSize):
    if not fileName or not os.path.exists(fileName):
        return
    fileSize = os.path.getsize(fileName)
    log.info('Отправляем объекты справочника в очередь...')
    jsonPiece = ''
    listObjects = []
    lastPercent = 0
    jloads = ujson.loads if version >= '2' else jsonWrapper

    with open(fileName, 'r') as dictFile:
        for line in dictFile:
            if line.strip() == "---":
                if jsonPiece.strip() != '':
                    listObjects.append(jloads(jsonPiece))
                jsonPiece = ''
            else:
                jsonPiece += ' ' + line
            if len(listObjects) == chunkSize:
                yield listObjects
                del listObjects[:]
                percent = dictFile.tell() * 100.0 / fileSize
                if (percent - lastPercent) > 10:
                    lastPercent = percent
                    log.info('Обработано %s%%' % int(percent))
        if len(listObjects) > 0:
            yield listObjects
            del listObjects[:]
            log.info('Обработано 100%')


def upload(fileName, dictProperties):
    dictCashLoadStateId = dictProperties['dictCashLoadStateId']
    version = dictProperties['version']

    if fileName and not os.path.isfile(fileName):
        mes = 'Созданный файл-справочник не найден.'
        log.error(mes)
        status.sendDictState(dictCashLoadStateId, 'Not found file-dictionary.', mes, status.STATUS_ERROR,
                             propertyReader.httpHostCS, propertyReader.httpPortCS, version)
        return status.errorProgramm, mes

    if not propertyReader.useQueueInUpload:
        return uploadWithoutQueue(fileName, dictProperties)

    # если пришла просто команда на генерацию справочника без самого справочника,
    # то не будем отправлять статус конвертирования
    isSendLoadDictStatus = dictProperties.get('nesGenerateCommandMode', 'none') == 'none'\
                           and not dictProperties.get('unloadElementName', None)
    dictCSLoadStateId = None
    if propertyReader.mode == 'server' and isSendLoadDictStatus:
        dictCSLoadStateId = status.sendDictStateForCS('All', 'Start putting dict to queue',
                                                      'Начинаем отправку справочника в очередь', status.STATUS_DURING)

    endTransProperties = {
        'host': 'localhost',
        'port': 3306,
        'username': 'netroot',
        'password': 'netroot',
        'dictMessageId': str(dictProperties['messageId']),  # uuid не пролазит в json
        'type': 'control',
        'control': 'end_tran',
        'dictBackOfficeId': dictCSLoadStateId,
        'dictCashLoadStateId': dictCashLoadStateId,
        'cashCode': propertyReader.cashCode,
        'statusServer': '%s:%s' % (propertyReader.remoteIp, propertyReader.cashServerRestPort),
        'version': version,
        'nesGenerateCommandMode': dictProperties['nesGenerateCommandMode'],
        'nesGenerateCommandCashesList': dictProperties['nesGenerateCommandCashesList'],
        'unloadElementName': dictProperties['unloadElementName'],
        'nesGenerateCommandOwner': dictProperties['nesGenerateCommandOwner'],
        'source': 'sync-core2'
        }

    queueName = "artix.loaddict.%s" % ("All" if propertyReader.mode == 'server' else 'cash')
    objectsCount = 0
    fileQueueManager = None
    transactionId = -1
    progress = Progress(log)
    try:
        fileQueueManager = FileQueueManager(propertyReader.fileQueueRoot, propertyReader.queueChunkSize, log, 'cs' if propertyReader.mode == 'server' else 'cash')
        fileQueueManager.removeUncompletedTransactions(queueName, 'sync-core2')
        log.info('Отправляем сообщение - начало транзакции')
        transactionId = fileQueueManager.beginTransaction(queueName)
        if progress:
            progress.createProgress(transactionId, objectsCount)

        for chunk in getAifChunk(version, fileName, fileQueueManager.chunkSize):
            objectsCount += len(chunk)
            fileQueueManager.enqueue(queueName, transactionId, {'properties': {'type': 'datagram'}, 'content': chunk})

        log.info('Отправляем сообщение - конец транзакции')
        endTransProperties['startMessageId'] = transactionId
        endTransProperties['objectsCount'] = objectsCount
        fileQueueManager.endTransaction(queueName, transactionId, {'properties': endTransProperties})
        if progress:
            progress.createProgress(transactionId, objectsCount)

        mes = 'Справочник поставлен в очередь на загрузку.'
        log.info(mes)
        if propertyReader.mode == 'server' and isSendLoadDictStatus:
            status.sendDictStateForCS('All', 'OK', 'OK', status.STATUS_READY, dictStateId=dictCSLoadStateId)
        status.sendDictState(dictCashLoadStateId, 'Send dictionary to queue for load to DB', mes, status.STATUS_DURING,
                             propertyReader.httpHostCS, propertyReader.httpPortCS, version)
        return status.OK, mes
    except Exception as e:
        if fileQueueManager:
            fileQueueManager.removeTransaction(queueName, transactionId)
        mes = 'Не удалось загрузить справочник. %s' % str(e)
        log.error(mes)
        if propertyReader.mode == 'server' and isSendLoadDictStatus:
            status.sendDictStateForCS('All', 'Can not load dictionary. %s' % str(e),
                                      mes, status.STATUS_ERROR, dictStateId=dictCSLoadStateId)
        status.sendDictState(dictCashLoadStateId, 'Can not load dictionary. %s' % str(e), mes, status.STATUS_ERROR,
                             propertyReader.httpHostCS, propertyReader.httpPortCS, version)
        if progress:
            progress.deleteProgress(transactionId)
        return status.errorLoadDict, mes


def sendCompareTableRequest(requestCash, compareName, compareResultServerId, resultServer=None):
    log.info(u'Отправляем сообщение для сверки данных <%s>' % compareName)
    endTransProperties = {
        'type': 'comparetable',
        'control': 'end_tran',
        'cashCode': propertyReader.cashCode,
        'statusServer': '%s:%s' % (propertyReader.remoteIp, propertyReader.cashServerRestPort),
        'requestCash': requestCash,
        'compareName': compareName,
        'compareResultServerId': compareResultServerId,
        'source': 'sync-core2'
        }
    queueName = "artix.loaddict.%s" % ("All" if propertyReader.mode == 'server' else 'cash')

    try:
        fileQueueManager = FileQueueManager(propertyReader.fileQueueRoot, propertyReader.queueChunkSize, log, 'cs' if propertyReader.mode == 'server' else 'cash')
        transactionId = fileQueueManager.beginTransaction(queueName)
        fileQueueManager.endTransaction(queueName, transactionId, {'properties': endTransProperties, 'content': resultServer})
    except Exception as e:
        mes = 'Не удалось отправить сообщение для сверки данных <%s>. %s' % (compareName, str(e))
        log.error(mes)
        return status.errorLoadDict, mes
    mes = 'Cообщение для сверки данных <%s> поставлено в очередь.' % compareName
    log.info(mes)
    return status.OK, mes
