# -*- coding: UTF-8 -*-

import ConfigParser
import infoReader
import logger
import os
import platform
import sys

loggerSync = logger.getLogger()
platformIsBionic = 'Ubuntu-18.04-bionic' in platform.platform()


def getValue(option, defaultValue, config):
    """
    Получить значение опции из конфигурационного файла.
    В случае ошибки вернуть значение по умолчанию
    """
    value = None
    try:
        if type(defaultValue) is int:
            value = config.getint('Properties', option)
        elif type(defaultValue) is bool:
            value = config.getboolean('Properties', option)
        else:
            value = config.get('Properties', option)
    except ConfigParser.NoOptionError:
        loggerSync.warn(u'Не найдена настройка %s. Берем значение по умолчанию: %s' % (option, defaultValue))
        value = defaultValue

    loggerSync.info(u'%s - %s' % (option.ljust(40, ' '), str(value).ljust(20, ' ')))
    return value


def getRequiredValue(option, config):
    """
    Получить обязательное значение из конф-го файла
    В случае, если нет секции или опции, генерируется Exception
    """
    value = config.get('Properties', option)
    loggerSync.info(u'%s - %s' % (option.ljust(40, ' '), str(value).ljust(20, ' ')))
    return value


def getPathProperties():
    return os.path.normpath('%s/../properties/cash.ini' % sys.path[0])


def rereadConfig():
    propertyReader = sys.modules[__name__]
    pathProperties = getPathProperties()
    # Считывание настроек подключения sync-агента
    if os.access(pathProperties, os.F_OK):
        config = ConfigParser.RawConfigParser()
        setattr(propertyReader, "config", config)
        config.read(pathProperties)
        setattr(propertyReader, "localBroker", getValue('localBroker', 'guest/guest@127.0.0.1:5672', config))
        remotePort = getValue('dept.broker.port', 5672, config)
        setattr(propertyReader, "remotePort", remotePort)
        setattr(propertyReader, "maxCountBackupFiles", getValue('maxCountBackupFiles', 0, config))
        setattr(propertyReader, "actualCountDaysForBackupFiles", getValue('actualCountDaysForBackupFiles', 0, config))
        cashServerRestPort = getValue('cashServerRestPort', 8080, config)
        setattr(propertyReader, "cashServerRestPort", cashServerRestPort)
        mode = getValue('mode', 'cash', config)
        setattr(propertyReader, "mode", mode)
        setattr(propertyReader, "uploadScript", getValue('uploadScript', '/linuxcash/cash/exchangesystems/exchangers/src/upload/converter/aif.py', config))
        setattr(propertyReader, "useQueueInUpload", getValue('useQueueInUpload', True, config))
        setattr(propertyReader, "brokerConnectionHeartbeat", getValue('brokerConnectionHeartbeat', 0, config))
        setattr(propertyReader, "dictUrlHostCS", getValue('dictUrlHostCS', '127.0.0.1', config))
        setattr(propertyReader, "dictUrlPortCS", getValue('dictUrlPortCS', 80, config))
        setattr(propertyReader, "clusterId", getValue('clusterId', '', config))
        setattr(propertyReader, "grpcHostCS", getValue('grpcHostCS', '127.0.0.1', config))
        setattr(propertyReader, "grpcPortCS", getValue('grpcPortCS', 10001, config))
        setattr(propertyReader, "grpcCaCertPath", getValue('grpcCaCertPath', None, config))
        protocolVersion = getValue('protocolVersion', 3, config)
        # в серверном режиме на ubunu 18.04 версия протокола не может быть меньше 5, т.к. на сервере отказались от qpid
        if mode == 'server' and (platformIsBionic or infoReader.package == 'artixcs-sync') and protocolVersion < 5:
            protocolVersion = 5
        setattr(propertyReader, "protocolVersion", protocolVersion)
        setattr(propertyReader, "unloadedSalesDirName", getValue('unloadedSalesDirName', 'unloadedSales', config))
        setattr(propertyReader, "unloadSalesRequestThroughCS", getValue('unloadSalesRequestThroughCS', False, config))
        try:
            # Значения, которые обязательно должны присутствовать в конф-ом файле
            remoteIp = getRequiredValue('dept.broker.ip', config)
            setattr(propertyReader, "remoteIp", remoteIp)
            setattr(propertyReader, "unloadScript", getRequiredValue('unloadScript', config))
            setattr(propertyReader, "cashCode", getRequiredValue('cash.code', config))
            setattr(propertyReader, "storeID", getRequiredValue('storeID', config))
            setattr(propertyReader, "remoteBroker", 'guest/guest@%s:%s' % (remoteIp, remotePort))
        except Exception as e:
            loggerSync.error(str(e))
            sys.exit()
        setattr(propertyReader, "httpHostCS",
                getValue('httpHostCS', remoteIp, config))  # почему-то параметр с КС передается, но на sync по обычаю игнорировался
        setattr(propertyReader, "httpPortCS",
                getValue('httpPortCS', cashServerRestPort, config))  # просто переименовал параметр, чтоб он совпадал со значением из SyncConfigInfo
        setattr(propertyReader, 'fileQueueRoot',
                getValue('fileQueueRoot', '/linuxcash/cash/data/queues' if mode == 'cash' else '/opt/queues', config))
        setattr(propertyReader, 'queueChunkSize', getValue('queueChunkSize', 25000, config))
    else:
        loggerSync.error(u'Нет файла с настройками')
        sys.exit()
        

rereadConfig()
